/************************************************************
**	File:	sample.c
**	Author:
**			Hasan Jamal
**
**	Description:
**		Sample application for various tests such as set/clear line signals, loopback, flush etc.
**    The sample application merely shows various features of the driver, it is not optimized
**    for any performance issues.
**    
**
**	Revision history:
**
**	Copyright (c) 2012, CTI, Connect Tech Inc. All Rights Reserved.
**
************************************************************/
//
//The sample is provided on an "AS IS" basis and Connect Tech Inc. makes absolutely 
//no warranty with respect to performance or the information contained 
//therein.  Connect Tech Inc. DISCLAIMS AND USER WAIVES ALL WARRANTIES, EXPRESS OR 
//IMPLIED, INCLUDING WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A 
//PARTICULAR PURPOSE.  The entire risk as to sample application quality and 
//performance is with the User.  ACCORDINGLY, IN NO EVENT SHALL Connect Tech Inc. 
//BE LIABLE FOR ANY DAMAGES, WHETHER IN CONTRACT OR TORT, INCLUDING 
//ANY LOST PROFITS OR OTHER INCIDENTAL, CONSEQUENTIAL, EXEMPLARY, OR 
//PUNITIVE DAMAGES ARISING OUT OF THE USE OR APPLICATION OF THE sample application.  
//Further, Connect Tech Inc. reserves the right to make changes to the sample application without 
//notice to improve reliability, function or design.  Connect Tech Inc. does not 
//convey any license under patent rights or any other intellectual 
//property rights, including those of third parties.
//
//

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <devctl.h>
#include <unistd.h>
#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/dcmd_chr.h>
#include <sys/select.h>



#include "CSPCI_pset.h"
#include "ioctls.h"

static char *version = "Comsync sample application version 1.00 April 09 2012\nConnect Tech Inc.\n\n";

#define FALSE 0
#define TRUE 1

#define MAX_CMD   20 //not a real number

#define RAW_MODE		1
#define FILE_MODE		2
#define LOOPBACK		3
#define COMMAND		4


//#define ASYNC
//#define MONO_SYNC
#define HDLC
//#define BI_SYNC
//#define USE_CRC

int goodPacket;
int dataMismatch;


// total bytes recevd
unsigned long recv_byte[2] = {0,0};
// total bytes sent
unsigned long send_byte[2] = {0,0};


#define RX	1
#define TX	2
#define TX_RX	3

#define CMD_USE "Command Usage: \n\
1 Set RTS\n\
2 Clear RTS\n\
3 Set DTR\n\
4 Clear DTR\n\
5 Set Break\n\
6 Clear Break\n\
7 Purge Rx\n\
8 Purge Tx\n\
9 Purge RxTx\n\
10 Get line state \n\
11 Transmit\n\
12 Receive\n\
13 loopback\n\
14 exit\n"



// buffer size
#define MAX_BUF_SIZE  4096
unsigned char  pakBuf[MAX_BUF_SIZE];

#define ESC	0x1b

struct _mode
{
	// mode for tansmit
	int tx;
	// mode for receive
	int rx;
};

struct _port_args {
	struct _mode mode;
	// packet size for receive and transmit
	int packetSize;
	// delay in ms before a read
	long delayRx;
	// delay in ms before a transmit
	long delayTx;
	// port index
	int port;
	// file descriptor for the port
	int fd;
	// file pointer for transmit/receive data
	FILE *fp;
	// flag to indicate send the file once or repeat
	char sendOnce;
	// flag to indicate whether non-blocking read/write to perform
	int non_block_io;
	// flag to indicate to use select api to wait for data
	int use_select;
	// flag to indicate to print events
	int show_events;
	// flag to indicate to exit thread
	int exit;
};



// parsing comsync.cfg file
void  configure_port(BHN_PortSettings *pset)
{		

	#ifdef ASYNC
	pset->smode.rx = SMODE_ASYNC;
	pset->smode.tx = SMODE_ASYNC;
	#endif

	#ifdef MONO_SYNC
	pset->smode.rx = SMODE_MONO;
	pset->smode.tx = SMODE_MONO;
	#endif

	#ifdef BI_SYNC
	pset->smode.rx = SMODE_BISYNC;
	pset->smode.tx = SMODE_BISYNC;
	#endif
	

	#ifdef HDLC
	pset->smode.rx = SMODE_HDLC;
	pset->smode.tx = SMODE_HDLC;
	#endif

	pset->line_mode = LM_V28;	

	pset->rxclk.enc_dec = NRZ;
	pset->rxclk.ref_freq = INT_REF_FREQ;
	pset->txclk.bps_error = 10000;
	pset->rxclk.bps = 9600;
	pset->rxclk.bps_frac = 0;

	#ifdef ASYNC
	pset->rxclk.async_div = 16;
	pset->rxclk.clk_tree.A = CLK_BRG0;	
	pset->rxclk.clk_tree.D = CTR_REF;	
	pset->txclk.async_div = 16;
	#else
	//pset->rxclk.clk_tree.A = CLK_RXC;
   	pset->rxclk.clk_tree.A = CLK_BRG1;
	pset->rxclk.clk_tree.C = BRG_CTR1;
	pset->rxclk.clk_tree.D = CTR_REF;
	pset->rxclk.clk_pin = XC_INP;
	#endif

	pset->txclk.ref_freq = INT_REF_FREQ;
	pset->txclk.bps_error = 10000;
	pset->txclk.bps = 9600;
	pset->txclk.bps_frac = 0;

	pset->txclk.dpll_div = 0;
	pset->txclk.ctr_div = 0;
	pset->txclk.enc_dec = NRZ;

	#ifdef ASYNC
	pset->txclk.clk_tree.A = CLK_BRG0;	
	pset->txclk.clk_tree.D = CTR_REF;		
	#else
	pset->txclk.clk_tree.A = CLK_BRG0;
	pset->txclk.clk_tree.C = BRG_CTR0;
	pset->txclk.clk_tree.D = CTR_REF;
	pset->txclk.clk_pin = TXC_TXCLK;
	pset->tx_pre_idle.tx_idle = IDLE_MARK;
	#endif

	#ifdef HDLC
		#ifdef USE_HDLC_ADDR
		pset->hdlc.rx_addr_control = HDLC_RX_ADDR1_CONT0;
		pset->sync_addr.rx0 = 'A';  // HDLC address
		#endif
	#endif

	pset->duplex_mode = DUPLEX_FULL;

	#ifdef MONO_SYNC
	pset->sync_addr.tx0 = 0x16;
	pset->sync_addr.tx1 = 0;
	pset->sync_addr.tx_len = 8;
	pset->sync_addr.rx0 = 0x16;
	pset->sync_addr.rx1 = 0;
	pset->sync_addr.rx_len = 8;
	pset->sync_addr.strip_sync = 1;
	#endif

	#ifdef BI_SYNC
	pset->sync_addr.tx0 = 0x16;
	pset->sync_addr.tx1 = 0x16;
	pset->sync_addr.tx_len = 8;
	pset->sync_addr.rx0 = 0x16;
	pset->sync_addr.rx1 = 0x16;
	pset->sync_addr.rx_len = 8;
	pset->sync_addr.strip_sync = 1;
	#endif

	

	/* Data bits (number of) */
	pset->dbits.tx = 8;
	pset->dbits.rx = 8;

	#ifdef ASYNC
	/* Async Mode Transmitter (fractional) 1 Stop bits */
	pset->tx_frac_stop = 16;
	/* Async flow control settings */
	pset->async_settings.xoff = 0x13;
	pset->async_settings.xon = 0x11;
	pset->async_settings.sflow = 'n';
	pset->async_settings.hflow = 0;
	#endif

	#ifdef USE_CRC
	/* TX CRC */
	pset->crc.tx_type = CCITT;
	pset->crc.tx_mode |= CRC_TX_EOFM;
	pset->crc.tx_start = CRC_START_1;
	/* RX CRC */
	pset->crc.rx_type = CCITT;
	pset->crc.rx_start = CRC_START_1;
		#ifdef USE_CRC_ON_UNDERRUN
		pset->crc.tx_mode |= CRC_TX_UNDERRUN;
		#else
		pset->hdlc.tx_underrun = HDLC_UNDERRUN_ABORT_7;
		#endif
	#else
	pset->crc.tx_type = CRC_NONE;
	pset->crc.rx_type = CRC_NONE;
		#ifdef HDLC
		pset->hdlc.tx_underrun = HDLC_UNDERRUN_FLAG;
		#endif
	#endif
		
}



// parameter input for receive, transmit or loopback
void GetDataOpParams(struct _port_args *Params, int mode)
{
	int val;
	// file name for transmit data
	char filename[100];
	char c;

	while(1)
	{
		printf("enter packet size (Max 4096)\n");
		scanf("%d",&val);
		if(val < 0 || val > 4096)
		{
			printf("wrong values\n");
			continue;
		}
		break;
	}
	Params->packetSize = val;

	if(Params->delayRx != -1)
	{
		while(1)
		{
			printf("enter delay between packets for receive (milliseconds)\n");
			scanf("%d",&val);
			if(val < 0)
			{
				printf("wrong values %d\n",val);
				continue;
			}
			break;
		}
		Params->delayRx = val;
	}
	if(Params->delayTx != -1)
	{
		while(1)
		{
			printf("enter delay between packets for transmit (milliseconds)\n");
			scanf("%d",&val);
			if(val < 0)
			{
				printf("wrong values %d\n",val);
				continue;
			}
			break;
		}
		Params->delayTx = val;
	}

	if(mode == TX_RX)
	{
		Params->mode.tx = LOOPBACK;
		Params->mode.rx = LOOPBACK;
	}
	else if(mode == RX)
	{
		while(1)
		{
			printf("enter data receive mode raw:1, file:2\n");
			scanf("%d",&val);
			if(val > 2 || val < 1)
			{
				printf("wrong values\n");
				continue;
			}
			break;
		}
		if(val == FILE_MODE)
		{
			while(1)
			{
				printf("enter file name\n");
				scanf("%s",filename);
				Params->fp = fopen(filename,"wt");
				if (!Params->fp)
				{
					printf("invalid filename\n");
				}
				else
				{
					break;
				}
			}
		}

		Params->mode.rx = val;

	}
	else if(mode == TX)
	{
		while(1)
		{
			printf("enter data transmit mode raw:1, file:2\n");
			scanf("%d",&val);
			if(val > 2 || val < 1)
			{
				printf("wrong values\n");
				continue;
			}
			break;
		}

		if(val == FILE_MODE)
		{
			while(1)
			{
				printf("enter file name\n");
				scanf("%s",filename);
				Params->fp = fopen(filename,"rt");
				if (!Params->fp)
				{
					printf("invalid filename\n");
					continue;
				}
				else
				{
					break;
				}
			}

			while(1)
			{
				printf("Send once? y/n\n");
				scanf("%c",&Params->sendOnce);
				if(Params->sendOnce == 'y' || Params->sendOnce == 'n')
					break;
			}
		}

		Params->mode.tx = val;
	}

}

// receive thread for each port
void *thread_read(void *arg) {
  int i,j, len, ret;
  // data buffer
  unsigned char pBuf[MAX_BUF_SIZE];
   struct _port_args *port_args = arg;
  // old running mode
  int oldMode;
  // current number bytes
  int curNumbytes = 0;
  // flag to indicate packet receive status
  unsigned char pakRecv = FALSE;
  // received raw data input buffer
  unsigned char inBuf[MAX_BUF_SIZE];
  // current position of receive raw data buffer
  int curPos = 0;
  // vars to use with select api
  fd_set tmpfds;
  fd_set readfds;
  int fdwidth;
  // events for TIOGETEVENT devctl
  unsigned long events=0;
  

  FD_ZERO(&readfds);
  FD_SET(port_args->fd, &readfds);
  fdwidth=1;
  fdwidth+=port_args->fd;

  Begin:
	while(port_args->mode.rx < 0 )
	{
		delay(1000);
		if (port_args->exit)
			return;
	}

	oldMode = port_args->mode.rx;

  while(1)
	{
		if(oldMode != port_args->mode.rx)
			goto Begin;

		if(port_args->use_select > 0)
		{
			select1:
			tmpfds = readfds;			
			i = select(fdwidth, &tmpfds, NULL, NULL, NULL);
			if(i <= 0)
				break;
			if (!FD_ISSET(port_args->fd, &tmpfds))				
			{
				goto select1;
			}
			
		}
		len = read(port_args->fd, pBuf, port_args->packetSize);
		//printf("read len %d\n",len);
		if(len < 0)
		{
			printf("read failed errno %d\n",errno);
			break;
		}
		if(len == 0)
			continue;
		curNumbytes +=len;
		if(curNumbytes >= port_args->packetSize)
			pakRecv = TRUE;
		recv_byte[port_args->port] += len;

		if(port_args->show_events > 0)
		{
			ret = devctl(port_args->fd, TIOGETEVENT, &events, sizeof(events), NULL);
			if (ret != 0) {
				printf("TIOGETEVENT failed ret %x\n", ret);
				exit(1);
			}
			if(events)
			{
				printf("events %x\n",events);
				events = 0;
			}
		}
		// print the raw data
		for(i=0;i<len;i++)
		{
			if((pBuf[i] == 0xff) || (pBuf[i] == 0x16))
			{
				//printf("TIOHUNT \n");
				ret = devctl(port_args->fd, TIOHUNT, NULL, 0, NULL);
				if (ret != 0) {
					printf("TIOSETPORTSET failed ret %x\n", ret);
					exit(1);
				}
				break;
			}

			if(port_args->mode.rx == FILE_MODE)
			{
				fwrite(&pBuf[i], 1, 1, port_args->fp);
				fflush(port_args->fp);
			}
			else if(port_args->mode.rx == LOOPBACK)
			{
				inBuf[curPos] = pBuf[i];
				curPos++;
				if(curPos == port_args->packetSize)
				{
					if(!memcmp(inBuf, pakBuf, port_args->packetSize))
						goodPacket++;
					else
					{
						dataMismatch++;
						for(j=0;j<port_args->packetSize; j++)
						{
							printf("%c%c",inBuf[j], pakBuf[j]);
							if(inBuf[j] != pakBuf[j])
							{
								printf("\nmismatch index %d %c%c 0x%x 0x%x\n",j, inBuf[j], pakBuf[j],inBuf[j], pakBuf[j]);
								//break;
								return 0;
							}
						}
					}

					curPos = 0;
				}
			}
			else if(port_args->mode.rx == RAW_MODE)
			{
				printf("%c",pBuf[i]);
				fflush(NULL);
			}
		}

		if(!pakRecv)
		{
			continue;
		}
		else
		{
			pakRecv = FALSE;
			curNumbytes = 0;
		}

		if(port_args->delayRx != -1)
		{
			delay(port_args->delayRx);
		}
	}

  return NULL;
}

// transmit thread for each port
void *thread_write(void *arg) {
	// data buffer
	char buf[MAX_BUF_SIZE];
	// old running mode
	int oldMode;
	 struct _port_args *port_args = arg;
	// total number of bytes transmitted
	unsigned long bytes;
	// retutrn value of an api
	int ret;
	// vars to use with select api
	fd_set tmpfds;
	fd_set writefds;
	int fdwidth;
	int bytes_sent_write;
	int mode = 0;
	
	FD_ZERO(&writefds);
	FD_SET(port_args->fd, &writefds);
	fdwidth=1;
	fdwidth+=port_args->fd;
	
	Begin:
	bytes = 0;
	while(port_args->mode.tx < 0)
	{
		delay(1000);
		if (port_args->exit)
			return;
	}

	oldMode = port_args->mode.tx;


	while(1) {
		if(oldMode != port_args->mode.tx){
			goto Begin;
		}
		if(port_args->mode.tx == FILE_MODE)
		{
			while(!feof(port_args->fp))
			{
				
				if(port_args->use_select > 0)
				{
					select1:
					tmpfds = writefds;
					ret = select(fdwidth, NULL, &tmpfds, NULL, NULL);
					if(ret <= 0)
						break;
					if (!FD_ISSET(port_args->fd, &tmpfds))				
					{
						goto select1;
					}
				}
		   
				ret = fread(buf, 1, port_args->packetSize, port_args->fp);
				bytes_sent_write = 0;
				while(bytes_sent_write != ret)
				{
					bytes_sent_write += write(port_args->fd, buf + bytes_sent_write, ret - bytes_sent_write);							
				}
				bytes += ret;
				if(port_args->delayTx != -1)
					delay(port_args->delayTx);
			}
			fseek(port_args->fp, SEEK_SET, SEEK_SET);
			if(port_args->sendOnce == 'y')
			{
				port_args->mode.tx = -1;
				fclose(port_args->fp);
				goto Begin;
			}
			bytes = 0;
		}
		else
		{
			if(port_args->use_select > 0)
			{
				select2:				
				tmpfds = writefds;
				ret = select(fdwidth, NULL, &tmpfds, NULL, NULL);
				if(ret <= 0)
				{
					printf("select_w failed\n");
					break;
				}
				if (!FD_ISSET(port_args->fd, &tmpfds))				
				{					
					goto select2;
				}		
					
			}
			
			ret = write(port_args->fd, pakBuf, port_args->packetSize);			
			
			bytes += ret;
			send_byte[port_args->port]=bytes;
			if(port_args->mode.tx != LOOPBACK)
				printf("\rsent %ld bytes", bytes);
			if(port_args->delayTx != -1)
					delay(port_args->delayTx);
		}
	}

  return NULL;
}

// initializes a port
int start_port( struct _port_args *port_args, BHN_PortSettings *pset, pthread_t *h_read, pthread_t *h_write, int port)
{

	char port_str[20];
	int ret;
	int mode = O_RDWR;
  	int ret_status;
	

	if(port_args->non_block_io)
		mode |= O_NONBLOCK;

	sprintf(port_str,"/dev/comsync%d",pset->port);
	port_args->fd = open(port_str, mode);
	if( port_args->fd == -1) {
		printf(" %s open error...\n",port_str);
		exit(1);
	}


	ret = devctl(port_args->fd, TIOSETPORTSET, pset, sizeof(BHN_PortSettings), &ret_status);
	if (ret != 0 || ret_status != 0) {
		printf("TIOSETPORTSET failed ret 0x%x\n", ret_status);
		exit(1);
	}

	if ( pthread_create( h_read, NULL, thread_read, port_args) ) {
	    printf("error creating thread.");
		exit(1);
	    
	}

	if ( pthread_create( h_write, NULL, thread_write, port_args) ) {
	    printf("error creating thread.");
	    exit(1);
	}

	return 1;
}

int main() {

	int  i;
	// port operation struct vars
	struct _port_args port_args1={0}, port_args2={0};
	// port configuration struct
	BHN_PortSettings pset1={0};
	BHN_PortSettings pset2={0};
	// pthread handles
	pthread_t h_read1;
	pthread_t h_write1;
	pthread_t h_read2;
	pthread_t h_write2;

	// command index
	int cmdID;
	// command string
	char cmdStr[100];
	// retutrn value of an api
	int ret;
	// set/reset value for commands
	unsigned char set_reset;
	// character input
	char c;
	
	
	printf("%s",version);
	

	while(1)
	{
		printf("Non-block io?y/n ");
		scanf("%c",&c);
		if(c == 'y')
		{
			port_args1.non_block_io = 1;
			break;
		}
		else if (c == 'n')
			break;
	}
	while(1)
	{
		printf("Print events?y/n ");
		scanf("%c",&c);
		if(c == 'y')
		{
			port_args1.show_events = 1;
			break;
		}
		else if (c == 'n')
			break;
	}
	while(1)
	{
		printf("Use select API?y/n ");
		scanf("%c",&c);
		if(c == 'y')
		{
			port_args1.use_select = 1;
			break;
		}
		else if (c == 'n')
			break;
	}


	memcpy(&port_args2, &port_args1, sizeof(port_args1));
	port_args1.mode.rx = -1;
	port_args1.mode.tx = -1;
	port_args2.mode.rx = -1;
	port_args2.mode.tx = -1;

	
	configure_port(&pset1);
	memcpy(&pset2, &pset1, sizeof(pset1));
	pset1.port = 0;	
	pset2.port = 1;
	start_port(&port_args1, &pset1, &h_read1, &h_write1, 0);
	
	
	for (i = 0; i < (int)MAX_BUF_SIZE; i++)
    {
        pakBuf[i] = (unsigned char) i % 26 + 0x41;
    }

	while(TRUE)
	{

		printf("%s",CMD_USE);

		scanf("%s",cmdStr);
		cmdID = atoi(cmdStr);
		if(cmdID == 14)
			goto Exit;

		switch(cmdID)
		{
		case 1:
			set_reset = 1;
			ret = devctl(port_args1.fd, TIOSETCLRRTS, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRRTS failed ret %x\n", ret);
				return 0;
			}
			break;
		case 2:
			set_reset = 0;
			ret = devctl(port_args1.fd, TIOSETCLRRTS, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRRTS failed ret %x\n", ret);
				return 0;
			}
			break;
		case 3:
			set_reset = 1;
			ret = devctl(port_args1.fd, TIOSETCLRDTR, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRDTR failed ret %x\n", ret);
				return 0;
			}
			break;
		case 4:
			set_reset = 0;
			ret = devctl(port_args1.fd, TIOSETCLRDTR, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRDTR failed ret %x\n", ret);
				return 0;
			}
			break;
		case 5:
			set_reset = 1;
			ret = devctl(port_args1.fd, TIOSETCLRBRK, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRBRK failed ret %x\n", ret);
				return 0;
			}
			break;
		case 6:
			set_reset = 0;
			ret = devctl(port_args1.fd, TIOSETCLRBRK, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRBRK failed ret %x\n", ret);
				return 0;
			}
			break;
		case 7:
			if(pset1.port > 2)
			{
				printf("starting port number %d (max is 2) exceeded for purge\n",
					pset1.port);
				continue;
			}
			port_args2.exit = 0;
			start_port(&port_args2, &pset2, &h_read2, &h_write2, 0);
			sleep(1);
			printf("\n\ndoing purge Rx between port %d and port %d \nflushing existing data\n\n",pset1.port+1,pset2.port+1);
			recv_byte[port_args1.port] = 0;
			send_byte[port_args2.port] = 0;
			port_args2.delayRx = 0;
			port_args2.delayTx = 0;
			port_args2.packetSize = 4096;
			port_args2.mode.tx = COMMAND;

			port_args1.delayRx = 0;
			port_args1.delayTx = 0;
			port_args1.packetSize = 4096;
			port_args1.mode.rx = COMMAND;
			sleep(2);
			ret = devctl(port_args1.fd, TCIFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCIFLUSH failed ret %x\n", ret);
				return 0;
         }
			port_args2.mode.tx = -1;
			port_args1.mode.rx = -1;
			sleep(2);
			printf("\nreceived %ld\n\n", recv_byte[port_args1.port]);
			if(recv_byte[port_args1.port] < send_byte[port_args2.port])
				printf("Purge Rx passed\n\n");
			else
				printf("Purge Rx failed\n\n");
			port_args2.exit = 1;
			close(port_args2.fd);
			sleep(1);
			break;

		case 8:
			if(pset1.port > 2)
			{
				printf("starting port number %d (max is 2) exceeded for purge\n",
					pset1.port);
				continue;
			}
			port_args2.exit = 0;
			start_port(&port_args2, &pset2, &h_read2, &h_write2, 0);
			sleep(1);
			printf("\n\ndoing purge Tx between port %d and port %d \nflushing existing data\n\n",pset1.port+1,pset2.port+1);
			recv_byte[port_args2.port] = 0;
			send_byte[port_args1.port] = 0;
			port_args2.delayRx = 0;
			port_args2.delayTx = 0;
			port_args2.packetSize = 4096;
			port_args2.mode.rx = COMMAND;

			port_args1.delayRx = 0;
			port_args1.delayTx = 0;
			port_args1.packetSize = 4096;
			port_args1.mode.tx = COMMAND;
			sleep(2);			
         ret = devctl(port_args1.fd, TCOFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCOFLUSH failed ret %x\n", ret);
				return 0;
         }
			port_args1.mode.tx = -1;
			port_args2.mode.rx = -1;
			sleep(2);
			printf("\nreceived %ld\n\n", recv_byte[port_args2.port]);
			if(recv_byte[port_args2.port] < send_byte[port_args1.port])
				printf("Purge Tx passed\n\n");
			else
				printf("Purge Tx failed\n\n");
			port_args2.exit = 1;
			close(port_args2.fd);
			sleep(1);
			break;

		case 9:
			if(pset1.port > 2)
			{
				printf("starting port number %d (max is 2) exceeded for purge\n",
					pset1.port);
				continue;
			}
			port_args2.exit = 0;
			start_port(&port_args2, &pset2, &h_read2, &h_write2, 0);
			sleep(1);
			printf("\n\ndoing purge Tx/Rx between port %d and port %d \nflushing existing data\n\n",pset1.port+1,pset2.port+1);
			recv_byte[port_args1.port] = 0;
			send_byte[port_args1.port] = 0;
			recv_byte[port_args2.port] = 0;
			send_byte[port_args2.port] = 0;
			port_args2.delayRx = 0;
			port_args2.delayTx = 0;
			port_args2.packetSize = 4096;
			port_args1.mode.rx = COMMAND;
			port_args2.mode.tx = COMMAND;

			port_args1.delayRx = 0;
			port_args1.delayTx = 0;
			port_args1.packetSize = 4096;
			port_args1.mode.rx = COMMAND;
			port_args1.mode.tx = COMMAND;
			sleep(2);
			
         ret = devctl(port_args1.fd, TCIOFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCIOFLUSH failed ret %x\n", ret);
				return 0;
         }
			port_args1.mode.tx = -1;
			port_args1.mode.rx = -1;
			port_args2.mode.tx = -1;
			port_args2.mode.rx = -1;
			sleep(2);
			printf("\nreceived %ld\n\n", recv_byte[port_args1.port]);
			if((recv_byte[port_args1.port] < send_byte[port_args2.port]) && (recv_byte[port_args2.port] < send_byte[port_args1.port]))
				printf("Purge Tx/Rx passed\n\n");
			else
				printf("Purge Tx/Rx failed\n\n");
			port_args2.exit = 1;
			close(port_args2.fd);
			sleep(1);
			break;

		case 10:
			ret = devctl(port_args1.fd, TIOCSERGETMSR, &set_reset, sizeof(set_reset), NULL);
			if (ret != 0) {
				printf("TIOSETCLRBRK failed ret %x\n", ret);
				return 0;
			}
			printf("MSR 0x%x\n",set_reset);
			break;

		case 11:
			ret = devctl(port_args1.fd, TCIOFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCIOFLUSH failed ret %x\n", ret);
				return 0;
         }
			port_args1.mode.rx = -1;
			port_args1.delayRx = -1;
			port_args1.delayTx = 0;			
			GetDataOpParams(&port_args1, TX);
			while(1)
			{
				if(port_args1.mode.tx < 0)
					break;
				sleep(1);
			}
			break;

		case 12:
			ret = devctl(port_args1.fd, TCIOFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCIOFLUSH failed ret %x\n", ret);
				return 0;
         }
			port_args1.mode.tx = -1;
			port_args1.delayTx = -1;
			port_args1.delayRx = 0;
			GetDataOpParams(&port_args1, RX);

			fflush(NULL);
			while(1)
			{
				if(port_args1.mode.rx < 0)
					break;
				sleep(1);
			}
			break;

		case 13:
			ret = devctl(port_args1.fd, TCIOFLUSH, NULL, 0, NULL);			
         if (ret != 0) {
            	printf("TCIOFLUSH failed ret %x\n", ret);
				return 0;
         }
         ret = devctl(port_args1.fd, TIOLOOPBACK, NULL, 0, NULL);
			if (ret != 0) {
				printf("TIOLOOPBACK failed ret %x\n", ret);
				exit(1);
			}

			GetDataOpParams(&port_args1, TX_RX);
			printf("Port  Totalpacket  Goodpacket  Datamismatch  \n");
			while(1)
			{
				sleep(1);

					printf("\r%4d%13ld%12ld%14ld", pset1.port+1,
						goodPacket + dataMismatch,
						goodPacket, dataMismatch);
					fflush(NULL);
					if(dataMismatch)
						return 0;
			}
			break;

		case ESC:
			break;
		default:
			printf("wrong command\n");
			break;
		}
	}

	close(port_args1.fd);

	Exit:

	return 0;
}
